@@ -1,6 +1,7 @@
 import logging
 
 from bitfield import BitField, BitHandler
+from django import db
 from django.conf import settings
 from django.db import transaction, connection
 from django.db.models import PositiveIntegerField, BigIntegerField, UniqueConstraint, Q
@@ -8,6 +9,7 @@ from django.forms import model_to_dict
 
 from isc_common import setAttr, delAttr
 from isc_common.bit import TurnBitOn
+from isc_common.common.functions import delete_dbl_spaces
 from isc_common.common.mat_views import refresh_mat_view
 from isc_common.fields.related import ForeignKeyProtect
 from isc_common.http.DSRequest import DSRequest
@@ -74,69 +76,139 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         return data
 
     @staticmethod
-    def delete_recursive(item_id, delete_lines=False, soft_delete=False, user=None, props=TurnBitOn(0, 0), document=None):
-        from django import db
+    def check_4_production(item):
+        if isinstance(item, int):
+            id = item
+        elif isinstance(item, Item):
+            id = item.id
+        else:
+            raise Exception(f'item must be Item or int')
+
+        with connection.cursor() as cursor:
+            cursor.execute(f'''select count(*) from ckk_item_not_typed_top_level_view
+                                where parent_id = %s or child_id = %s''', [id, id])
+            qty, = cursor.fetchone()
+            if qty == 0:
+                cursor.execute(f'''select count(*) from ckk_item_typed_top_level_view
+                                    where parent_id = %s or child_id = %s''', [id, id])
+                qty, = cursor.fetchone()
+            return qty > 0
+
+    @staticmethod
+    def delete_recursive(item_id, delete_lines=False, soft_delete=False, user=None, props=TurnBitOn(0, 0), document=None, show_progress=True):
         from isc_common.models.deleted_progresses import Deleted_progresses
         from kaf_pas.ckk.models.item_image_refs import Item_image_refs
         from kaf_pas.ckk.models.item_line import Item_line
         from kaf_pas.ckk.models.item_refs import Item_refs
         from kaf_pas.production.models.operations_item import Operations_item
+        from kaf_pas.ckk.models.item_document import Item_document
 
         settings.LOCKS.acquire('ItemManager.delete_recursive')
         item_lotsman_document_cnt = 0
-        cnt = Item_refs.objects.get_descendants_count(id=item_id)
+        if document != None:
+            if ItemManager.check_4_production(item=item_id) == True:
+                settings.LOCKS.release('ItemManager.delete_recursive')
+                return 2
 
-        with managed_progress(
-                id=f'delete_recursive_{item_id}',
-                qty=cnt,
-                user=user,
-                message='Удаление товарной(ых) позиций',
-                title='Выполнено',
-                props=props
-        ) as progress:
+        sql_text = None
+        if isinstance(document, Documents):
+            sql_text = f'''WITH RECURSIVE r AS (
+                                                    SELECT cir.*, 1 AS level
+                                                    FROM ckk_item_refs cir
+                                                             join ckk_item cich on cich.id = cir.child_id
+                                                    WHERE cir.child_id IN ({item_id})
+                                                      and cich.document_id = {document.id}
+                                                
+                                                    union all
+                                                
+                                                    SELECT cir.*, r.level + 1 AS level
+                                                    FROM ckk_item_refs cir
+                                                             join ckk_item cich on cich.id = cir.child_id
+                                                             JOIN r ON cir.parent_id = r.child_id
+                                                    WHERE cich.document_id = {document.id}
+                                                )
+                                                
+                                                select *
+                                                from r'''
+        elif isinstance(document, Lotsman_documents_hierarcy):
+            sql_text = f'''WITH RECURSIVE r AS (
+                                                    SELECT cir.*, 1 AS level
+                                                    FROM ckk_item_refs cir
+                                                             join ckk_item cich on cich.id = cir.child_id
+                                                    WHERE cir.child_id IN ({item_id})
+                                                      and cich.lotsman_document_id = {document.id}
 
+                                                    union all
+
+                                                    SELECT cir.*, r.level + 1 AS level
+                                                    FROM ckk_item_refs cir
+                                                             join ckk_item cich on cich.id = cir.child_id
+                                                             JOIN r ON cir.parent_id = r.child_id
+                                                    WHERE cich.lotsman_document_id = {document.id}
+                                                )
+
+                                                select *
+                                                from r'''
+
+        cnt = Item_refs.objects.get_descendants_count(id=item_id, sql_text=sql_text)
+
+        def _delete(progress=None):
             try:
                 with transaction.atomic():
                     try:
                         item = Item.objects.get(id=item_id)
                         if cnt == 0:
                             Item_refs.objects1.filter(parent=item).delete_with_checked()
+                            Item_line.objects.filter(parent=item).delete()
                             item.delete()
-                            settings.LOCKS.release('ItemManager.delete_recursive')
-                            return 1
+                        else:
+                            _item = None
+                            if progress != None:
+                                progress.setContentsLabel(f'''<h3>Удаление товарной позиции ({item.STMP_1.value_str if item.STMP_1 else ''} : {item.STMP_2.value_str if item.STMP_2 else ''}).</h3>''')
 
-                        _item = None
-                        progress.setContentsLabel(f'''<h3>Удаление товарной позиции ({item.STMP_1.value_str if item.STMP_1 else ''} : {item.STMP_2.value_str if item.STMP_2 else ''}).</h3>''')
-                        for item in Item_refs.objects.get_descendants(id=item_id, order_by_clause='order by level desc'):
-                            _item = item
-                            if not soft_delete:
-                                Item_image_refs.objects.filter(item_id=item.child_id).delete()
+                            for item in Item_refs.objects.get_descendants(id=item_id, order_by_clause='order by level desc', sql_text=sql_text):
+                                _item = item
+                                _continue = False
+                                if not soft_delete:
+                                    if document != None:
+                                        if ItemManager.check_4_production(item=item.child_id) == True:
+                                            _continue=True
 
-                                if delete_lines:
-                                    Item_line.objects.filter(parent_id=item.child_id).delete()
-                                    Item_line.objects.filter(child_id=item.child_id).delete()
+                                    if _continue:
+                                        _continue=False
+                                        continue
 
-                                Item_refs.objects1.filter(parent_id=item.child_id).delete_with_checked()
-                                Item_refs.objects1.filter(child_id=item.child_id).delete_with_checked()
+                                    Item_image_refs.objects.filter(item_id=item.child_id).delete()
 
-                                Operations_item.objects.filter(item=item.child_id).delete()
-                                # Ready_2_launch_detail.objects.filter(item=item.child_id).delete()
+                                    if delete_lines:
+                                        Item_line.objects.filter(parent_id=item.child_id).delete()
+                                        Item_line.objects.filter(child_id=item.child_id).delete()
 
-                                Item.objects.filter(id=item.child_id).delete()
-                            else:
-                                Item_image_refs.objects.filter(item_id=item.child_id).soft_delete()
+                                    Item_refs.objects1.filter(parent_id=item.child_id).delete_with_checked()
+                                    Item_refs.objects1.filter(child_id=item.child_id).delete_with_checked()
 
-                                if delete_lines:
-                                    Item_line.objects.filter(parent_id=item.child_id).soft_delete()
-                                    Item_line.objects.filter(child_id=item.child_id).soft_delete()
+                                    Operations_item.objects.filter(item=item.child_id).delete()
+                                    # Ready_2_launch_detail.objects.filter(item=item.child_id).delete()
 
-                                Item_refs.objects.filter(parent_id=item.child_id).soft_delete()
-                                Item_refs.objects.filter(child_id=item.child_id).soft_delete()
-                                Item.objects.filter(id=item.child_id).soft_delete()
+                                    Item_document.objects.filter(item_id=item.child_id).delete()
+                                    Item.objects.filter(id=item.child_id).delete()
+                                else:
+                                    Item_image_refs.objects.filter(item_id=item.child_id).soft_delete()
 
-                            step_res = progress.step()
-                            if step_res != 0:
-                                raise ProgressDroped(progress_deleted)
+                                    if delete_lines:
+                                        Item_line.objects.filter(parent_id=item.child_id).soft_delete()
+                                        Item_line.objects.filter(child_id=item.child_id).soft_delete()
+
+                                        Item_line.objects.filter(parent_id=item.parent_id).soft_delete()
+                                        Item_line.objects.filter(child_id=item.parent_id).soft_delete()
+
+                                    Item_refs.objects.filter(parent_id=item.child_id).soft_delete()
+                                    Item_refs.objects.filter(child_id=item.child_id).soft_delete()
+                                    Item.objects.filter(id=item.child_id).soft_delete()
+
+                                if progress != None:
+                                    if  progress.step() != 0:
+                                        raise ProgressDroped(progress_deleted)
                     except Item.DoesNotExist:
                         settings.LOCKS.release('ItemManager.delete_recursive')
 
@@ -150,6 +222,19 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
                 settings.LOCKS.release('ItemManager.delete_recursive')
                 raise ex
 
+        if show_progress and cnt > 0:
+            with managed_progress(
+                    id=f'delete_recursive_{item_id}',
+                    qty=cnt,
+                    user=user,
+                    message='Удаление товарной(ых) позиций',
+                    title='Выполнено',
+                    props=props
+            ) as progress:
+               _delete(progress)
+        else:
+            _delete()
+
         settings.LOCKS.release('ItemManager.delete_recursive')
 
         if item_lotsman_document_cnt > 0:
@@ -372,15 +457,15 @@ class ItemManager(CommonManagetWithLookUpFieldsManager):
         settings.LOCKS.acquire('ItemManager.get_verstion')
         try:
             res = Item.objects.filter(
-                STMP_1_id=STMP_1.id if STMP_1 else None,
-                STMP_2_id=STMP_2.id if STMP_2 else None,
+                STMP_1__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_1.value_str) if STMP_1 else None,
+                STMP_2__value_str__delete_dbl_spaces=delete_dbl_spaces(STMP_2.value_str) if STMP_2 else None,
                 props=props,
                 version__isnull=False
             ).order_by('-version')[0].version + inc
             if res == None:
                 res = 1
         except IndexError:
-            res = 1
+            res = None
         settings.LOCKS.release('ItemManager.get_verstion')
         return res
 
