@@ -53,6 +53,7 @@ class Item_line_viewManager(AuditManager):
             'deliting': record.deliting,
             'item_props': record.item_props._value,
             'relevant': record.relevant,
+            'confirmed': record.confirmed,
         }
         return res
 
@@ -80,6 +81,7 @@ class Item_line_view(AuditModel):
     where_from = NameField()
     item_props = Item_add.get_prop_field()
     relevant = NameField()
+    confirmed = NameField()
 
     objects = Item_line_viewManager()
 
